#ifndef __ATMEL_SAM7A3EK_H__
#define __ATMEL_SAM7A3EK_H__
/*====================================================================
* Project: Board Support Package (BSP)
* Developed using:
* Function: Frequencies on Atmel-SAM7SA3EK board
*
* Copyright HighTec EDV-Systeme GmbH 1982-2006
*====================================================================*/


#include "at91sam7a3.h"

#define FREQ_OSC		18423000	/* External oscillator frequency */
#define FREQ_PLL		95953125	/* PLL output frequency (MUL=124,DIV=24) */
#define FREQ_MCK		47976562	/* CPU frequency */


#endif /* __ATMEL_SAM7A3EK_H__ */
