/*
 * This file is part of the Code::Blocks IDE and licensed under the GNU Lesser General Public License, version 3
 * http://www.gnu.org/licenses/lgpl-3.0.html
 */

#ifndef LICENSES_H
#define LICENSES_H

// license macros
#define LICENSE_GPL _("This program is licensed under the terms\nof the GNU General Public License version 3\n\n" \
  "Available online under:\nhttp://www.gnu.org/licenses/gpl-3.0.html")

#define LICENSE_WXWINDOWS _("This program is distributed under the wxWindows Library License version 3\n\n" \
  "http://www.wxwidgets.org/about/licence3.txt")

#endif // LICENSES_H

